from rest_framework import serializers

from core.models import (Submission, SubmissionSubscription,
                         TutorSubmissionAssignment)
from core.serializers import (DynamicFieldsModelSerializer, FeedbackSerializer,
                              TestSerializer)


class SubmissionAssignmentSerializer(DynamicFieldsModelSerializer):
    text = serializers.ReadOnlyField()
    type = serializers.ReadOnlyField(source='type.pk')
    full_score = serializers.ReadOnlyField(source='type.full_score')
    tests = TestSerializer(many=True, read_only=True)

    class Meta:
        model = Submission
        fields = ('pk', 'type', 'text', 'full_score', 'tests')


class AssignmentSerializer(DynamicFieldsModelSerializer):
    owner = serializers.StringRelatedField(source='subscription.owner')
    stage = serializers.StringRelatedField(
        source='subscription.feedback_stage')

    class Meta:
        model = TutorSubmissionAssignment
        fields = ('pk', 'submission', 'is_done', 'owner', 'stage')
        read_only_fields = ('is_done', 'submission')


class AssignmentDetailSerializer(AssignmentSerializer):
    feedback = FeedbackSerializer(source='submission.feedback', read_only=True)
    submission = SubmissionAssignmentSerializer(read_only=True)

    class Meta:
        model = TutorSubmissionAssignment
        fields = ('pk', 'submission', 'feedback', 'is_done', 'subscription',
                  'owner', 'stage')
        read_only_fields = ('is_done', 'submission', 'feedback')

    def create(self, validated_data):
        subscription = validated_data.get('subscription')
        return subscription.get_or_create_work_assignment()


class SubscriptionSerializer(DynamicFieldsModelSerializer):
    owner = serializers.ReadOnlyField(source='owner.username')
    query_key = serializers.UUIDField(required=False)
    assignments = serializers.SerializerMethodField()
    remaining = serializers.SerializerMethodField()
    available = serializers.SerializerMethodField()

    def get_assignments(self, subscription):
        queryset = subscription.assignments.filter(is_done=False)
        serializer = AssignmentDetailSerializer(queryset, many=True)
        return serializer.data

    def get_remaining(self, subscription):
        return subscription.get_remaining_not_final()

    def get_available(self, subscription):
        return subscription.get_available_in_stage()

    def validate(self, data):
        data['owner'] = self.context['request'].user

        if 'query_key' in data != \
                data['query_type'] == SubmissionSubscription.RANDOM:
            raise serializers.ValidationError(
                f'The {data["query_type"]} query_type does not work with the'
                f'provided key')

        elif 'query_key' in data:
            query_type = data.get('query_type')
            query_key = data.get('query_key')

            select = SubmissionSubscription.type_query_mapper[query_type]
            if Submission.objects.filter(**{select: query_key}).count() == 0:
                raise serializers.ValidationError(
                    'Seems no submissions exist for given query and key')

        return data

    def create(self, validated_data) -> SubmissionSubscription:
        return SubmissionSubscription.objects.create(**validated_data)

    def update(self, instance, validated_data):
        instance.deactivated = False
        instance.save()
        return instance

    class Meta:
        model = SubmissionSubscription
        fields = ('pk',
                  'owner',
                  'query_type',
                  'query_key',
                  'feedback_stage',
                  'deactivated',
                  'assignments',
                  'remaining',
                  'available')
        read_only_fields = ('deactivated',)
