from django.conf import settings
from rest_framework.test import APITestCase

from core.models import Feedback, SubmissionSubscription
from util.factories import make_test_data


class StopAfterFiftyPercent(APITestCase):

    @classmethod
    def setUpTestData(cls):
        settings.STOP_ON_PASS = True

    def setUp(self):
        self.data = make_test_data(data_dict={
            'exams': [{
                'module_reference': 'Test Exam 01',
                'total_score': 50,
                'pass_score': 25,
            }],
            'submission_types': [
                {
                    'name': '01. Sort this or that',
                    'full_score': 20,
                    'description': 'Very complicated',
                    'solution': 'Trivial!'
                },
                {
                    'name': '02. Merge this or that or maybe even this',
                    'full_score': 15,
                    'description': 'Very complicated',
                    'solution': 'Trivial!'
                },
                {
                    'name': '03. Super simple task',
                    'full_score': 15,
                    'description': 'Very complicated',
                    'solution': 'Trivial!'
                },
            ],
            'students': [
                {'username': 'student01', 'exam': 'Test Exam 01'},
            ],
            'tutors': [
                {'username': 'tutor01'},
                {'username': 'tutor02'}
            ],
            'reviewers': [
                {'username': 'reviewer'}
            ],
            'submissions': [
                {
                    'text': 'function blabl\n'
                            '   on multi lines\n'
                            '       for blabla in bla:\n'
                            '           lorem ipsum und so\n',
                    'type': '01. Sort this or that',
                    'user': 'student01',
                },
                {
                    'text': 'function blabl\n'
                            '       asasxasx\n'
                            '           lorem ipsum und so\n',
                    'type': '02. Merge this or that or maybe even this',
                    'user': 'student01'
                },
                {
                    'text': 'function blabl\n'
                            '   on multi lines\n'
                            '       asasxasx\n'
                            '           lorem ipsum und so\n',
                    'type': '03. Super simple task',
                    'user': 'student01'
                }
            ]}
        )
        self.tutor01 = self.data['tutors'][0]
        self.tutor02 = self.data['tutors'][1]
        self.subscription = SubmissionSubscription.objects.create(
            owner=self.tutor01,
            feedback_stage=SubmissionSubscription.FEEDBACK_CREATION)

    def test_all_feedback_is_available(self):
        self.assertEqual(3, self.subscription.get_available_in_stage())

    def test_all_is_available_when_score_is_too_low(self):
        Feedback.objects.create(
            of_submission=self.data['submissions'][0], score=20, is_final=True)
        self.assertEqual(2, self.subscription.get_available_in_stage())

    def test_default_does_not_pass_exam(self):
        self.assertFalse(self.data['students'][0].student.passes_exam)

    def test_no_more_submissions_after_student_passed_exam(self):
        Feedback.objects.create(
            of_submission=self.data['submissions'][0], score=20)
        Feedback.objects.create(
            of_submission=self.data['submissions'][1], score=15)

        self.data['students'][0].student.refresh_from_db()
        self.assertEqual(0, self.subscription.get_available_in_stage())
        self.assertEqual(35, self.data['students'][0].student.total_score)
        self.assertTrue(self.data['students'][0].student.passes_exam)

    def test_validation_still_allowed(self):
        a1 = self.subscription.get_or_create_work_assignment()
        a2 = self.subscription.get_or_create_work_assignment()

        # signals recognize the open assignments
        Feedback.objects.create(
            of_submission=a1.submission, score=20)
        Feedback.objects.create(
            of_submission=a2.submission, score=15)

        subscription = SubmissionSubscription.objects.create(
            owner=self.tutor02,
            feedback_stage=SubmissionSubscription.FEEDBACK_VALIDATION)

        self.assertEqual(0, self.subscription.get_available_in_stage())
        self.assertEqual(2, subscription.get_available_in_stage())
        self.assertEqual(3, subscription.get_remaining_not_final())
