import Vue from 'vue'
import Router, {RawLocation, Route, NavigationGuard} from 'vue-router'
import Login from '@/pages/Login.vue'
import StudentSubmissionPage from '@/pages/student/StudentSubmissionPage.vue'
import StudentOverviewPage from '@/pages/reviewer/StudentOverviewPage.vue'
import TutorOverviewPage from '@/pages/reviewer/TutorOverviewPage.vue'
import SubscriptionWorkPage from '@/pages/SubscriptionWorkPage.vue'
import SubscriptionEnded from '@/components/subscriptions/SubscriptionEnded.vue'
import PageNotFound from '@/pages/PageNotFound.vue'
import StartPageSelector from '@/pages/StartPageSelector.vue'
import LayoutSelector from '@/pages/LayoutSelector.vue'
import StudentSubmissionSideView from '@/pages/StudentSubmissionSideView.vue'
import StudentListHelpCard from '@/components/student_list/StudentListHelpCard.vue'
import FeedbackHistoryPage from '@/pages/base/FeedbackHistoryPage.vue'
import FeedbackTable from '@/components/feedback_list/FeedbackTable.vue'
import FeedbackListHelpCard from '@/components/feedback_list/FeedbackListHelpCard.vue'
import VueInstance from '@/main'
import store from '@/store/store'

Vue.use(Router)

type rerouteFunc = (to?: RawLocation | false | ((vm: Vue) => any) | void) => void

function denyAccess (next: rerouteFunc, redirect: Route) {
  next(redirect.path)
  VueInstance.$notify({
    title: 'Access denied',
    text: "You don't have permission to view this.",
    type: 'error'
  })
}

let tutorOrReviewerOnly: NavigationGuard = function (to, from, next) {
  if (store.getters.isTutorOrReviewer) {
    next()
  } else {
    denyAccess(next, from)
  }
}

let reviewerOnly: NavigationGuard = function (to, from, next) {
  if (store.getters.isReviewer) {
    next()
  } else {
    denyAccess(next, from)
  }
}

let studentOnly: NavigationGuard = function (to, from, next) {
  if (store.getters.isStudent) {
    next()
  } else {
    next(false)
  }
}

let checkLoggedIn: NavigationGuard = function (to, from, next) {
  if (store.getters.isLoggedIn) {
    next()
  } else {
    next('/login/')
  }
}

const router = new Router({
  routes: [
    {
      path: '/login/',
      name: 'login',
      component: Login
    },
    {
      path: '',
      redirect: 'home',
      beforeEnter: checkLoggedIn,
      component: LayoutSelector,
      children: [
        {
          path: 'home',
          name: 'home',
          component: StartPageSelector
        },
        {
          path: 'subscription/ended',
          name: 'subscription-ended',
          component: SubscriptionEnded
        },
        {
          path: 'subscription/:pk',
          name: 'subscription',
          beforeEnter: tutorOrReviewerOnly,
          component: SubscriptionWorkPage
        },
        {
          path: 'feedback',
          beforeEnter: tutorOrReviewerOnly,
          component: FeedbackHistoryPage,
          children: [
            {
              path: '',
              name: 'feedback',
              components: {
                left: FeedbackTable,
                right: FeedbackListHelpCard
              }
            },
            {
              path: ':submissionPk',
              components: {
                left: FeedbackTable,
                right: StudentSubmissionSideView
              },
              meta: {
                submissionSideView: true
              }
            },
            {
              path: ':id/detail',
              components: {
                left: FeedbackTable,
                right: PageNotFound
              }
            }
          ]
        },
        {
          path: 'student-overview',
          beforeEnter: reviewerOnly,
          component: StudentOverviewPage,
          children: [
            {
              path: '',
              name: 'student-overview',
              component: StudentListHelpCard
            },
            {
              path: 'student/:studentPk/submission/:submissionPk',
              name: 'submission-side-view',
              component: StudentSubmissionSideView,
              meta: {
                submissionSideView: true
              }
            }
          ]
        },
        {
          path: 'tutor-overview',
          name: 'tutor-overview',
          beforeEnter: reviewerOnly,
          component: TutorOverviewPage
        },
        {
          path: 'submission/:id',
          beforeEnter: studentOnly,
          component: StudentSubmissionPage
        }
      ]
    },
    {
      path: '*',
      name: 'page-not-found',
      component: PageNotFound
    }
  ]
})

export default router
