import * as api from '@/api'
import gradySays from '../grady_speak'

function initialState () {
  return {
    token: sessionStorage.getItem('token'),
    lastTokenRefreshTry: Date.now(),
    refreshingToken: false,
    jwtTimeDelta: 0,
    message: '',
    user: {
      pk: '',
      username: '',
      role: '',
      is_admin: ''
    }
  }
}

export const authMut = Object.freeze({
  SET_MESSAGE: 'SET_MESSAGE',
  SET_JWT_TOKEN: 'SET_JWT_TOKEN',
  SET_JWT_TIME_DELTA: 'SET_JWT_TIME_DELTA',
  SET_USER: 'SET_USER',
  SET_LAST_TOKEN_REFRESH_TRY: 'SET_LAST_TOKEN_REFRESH_TRY',
  RESET_STATE: 'RESET_STATE',
  SET_REFRESHING_TOKEN: 'SET_REFRESHING_TOKEN'
})

const authentication = {
  state: initialState(),
  getters: {
    gradySpeak: () => {
      return gradySays[Math.floor(Math.random() * gradySays.length)]
    },
    isStudent: state => {
      return state.user.role === 'Student'
    },
    isTutor: state => {
      return state.user.role === 'Tutor'
    },
    isReviewer: state => {
      return state.user.role === 'Reviewer'
    },
    isTutorOrReviewer: (state, getters) => {
      return getters.isTutor || getters.isReviewer
    },
    isLoggedIn: state => !!state.token
  },
  mutations: {
    [authMut.SET_MESSAGE] (state, message) {
      state.message = message
    },
    [authMut.SET_JWT_TOKEN] (state, token) {
      sessionStorage.setItem('token', token)
      state.token = token
    },
    [authMut.SET_JWT_TIME_DELTA] (state, timeDelta) {
      state.jwtTimeDelta = timeDelta
    },
    [authMut.SET_USER] (state, user) {
      state.user = user
    },
    [authMut.SET_REFRESHING_TOKEN] (state, refreshing) {
      state.refreshingToken = refreshing
    },
    [authMut.SET_LAST_TOKEN_REFRESH_TRY] (state) {
      state.lastTokenRefreshTry = Date.now()
    },
    [authMut.RESET_STATE] (state) {
      sessionStorage.setItem('token', '')
      Object.assign(state, initialState())
    }
  },
  actions: {
    async getJWT (context, credentials) {
      try {
        const token = await api.fetchJWT(credentials)
        context.commit(authMut.SET_JWT_TOKEN, token)
      } catch (error) {
        let errorMsg
        if (!error.response) {
          errorMsg = 'Cannot reach server.'
        } else if (error.response.status === 400) {
          errorMsg = 'Unable to log in with provided credentials.'
        } else if (error.response.status === 429) {
          errorMsg = error.response.data.detail
        }
        context.commit(authMut.SET_MESSAGE, errorMsg)
        throw errorMsg
      } finally {
        context.commit(authMut.SET_LAST_TOKEN_REFRESH_TRY)
      }
    },
    async refreshJWT ({state, commit}) {
      commit(authMut.SET_REFRESHING_TOKEN, true)
      try {
        const token = await api.refreshJWT(state.token)
        commit(authMut.SET_JWT_TOKEN, token)
      } finally {
        commit(authMut.SET_REFRESHING_TOKEN, false)
        commit(authMut.SET_LAST_TOKEN_REFRESH_TRY)
      }
    },
    async getUser ({commit}) {
      try {
        const user = await api.getOwnUser()
        commit(authMut.SET_USER, user)
      } catch (err) {
        commit(authMut.SET_MESSAGE, 'Unable to fetch user.')
      }
    },
    async getJWTTimeDelta ({commit}) {
      try {
        const delta = await api.fetchJWTTimeDelta()
        // multiply by 1000 to convert to ms
        commit(authMut.SET_JWT_TIME_DELTA, delta * 1000)
      } catch (err) {
        console.log(err)
      }
    }
  }
}

export default authentication
