import Vue from 'vue'
import * as api from '@/api'
import {handleError, flatten, cartesian, once} from '@/util/helpers'

export const subscriptionMuts = Object.freeze({
  SET_SUBSCRIPTIONS: 'SET_SUBSCRIPTIONS',
  SET_SUBSCRIPTION: 'SET_SUBSCRIPTION',
  SET_LOADING: 'SET_LOADING',
  SET_ACTIVE_SUBSCRIPTION_PK: 'SET_ACTIVE_SUBSCRIPTION_PK',
  SET_ASSIGNMENT_QUEUE: 'SET_ASSIGNMENT_QUEUE',
  ADD_ASSIGNMENT_TO_QUEUE: 'ADD_ASSIGNMENT_TO_QUEUE',
  POP_ASSIGNMENT_FROM_QUEUE: 'POP_ASSIGNMENT_FROM_QUEUE',
  RESET_STATE: 'RESET_STATE'
})

function initialState () {
  return {
    subscriptions: {},
    assignmentQueue: [],
    activeSubscriptionPk: '',
    loading: false
  }
}

const MAX_NUMBER_OF_ASSIGNMENTS = 2

// noinspection JSCommentMatchesSignature
const subscriptions = {
  state: initialState(),
  getters: {
    availableTypes (state, getters) {
      let types = ['random', 'submission_type']
      if (getters.isReviewer) {
        types.push('exam')
      }
      return types
    },
    availableStages (state, getters) {
      let stages = ['feedback-creation', 'feedback-validation']
      if (getters.isReviewer) {
        stages.push('feedback-conflict-resolution')
      }
      return stages
    },
    availableStagesReadable (state, getters) {
      let stages = ['create', 'validate']
      if (getters.isReviewer) {
        stages.push('resolve')
      }
      return stages
    },
    availableSubmissionTypeQueryKeys (state, getters, rootState) {
      return Object.values(rootState.submissionTypes).map(subType => subType.pk)
    },
    availableExamTypeQueryKeys (state, getters, rootState) {
      return Object.values(rootState.examTypes).map(examType => examType.pk)
    },
    activeSubscription (state) {
      return state.subscriptions[state.activeSubscriptionPk]
    },
    resolveSubscriptionKeyToName: (state, getters, rootState) => subscription => {
      switch (subscription.query_type) {
        case 'random':
          return 'Active'
        case 'exam':
          const examType = rootState.examTypes[subscription.query_key]
          return examType ? examType.module_reference : 'Exam'
        case 'submission_type':
          const submissionType = rootState.submissionTypes[subscription.query_key]
          return submissionType ? submissionType.name : 'Submission Type'
        case 'student':
          const studentName = rootState.students[subscription.query_key]
          return studentName ? studentName.name : 'Student'
      }
    },
    getSubscriptionsGroupedByType (state, getters) {
      const subscriptionsByType = () => {
        return {
          'random': [],
          'student': [],
          'exam': [],
          'submission_type': []
        }
      }
      let subscriptionsByStage = getters.availableStages.reduce((acc, curr) => {
        acc[curr] = subscriptionsByType()
        return acc
      }, {})
      Object.values(state.subscriptions).forEach(subscription => {
        subscriptionsByStage[subscription.feedback_stage][subscription.query_type].push(subscription)
      })
      // sort the resulting arrays in subscriptions lexicographically by their query_keys
      const sortSubscriptions = subscriptionsByType => Object.values(subscriptionsByType).forEach(arr => {
        if (arr.length > 1 && arr[0].hasOwnProperty('query_key')) {
          arr.sort((subA, subB) => {
            const subALower = getters.resolveSubscriptionKeyToName(subA).toLowerCase()
            const subBLower = getters.resolveSubscriptionKeyToName(subB).toLowerCase()
            if (subALower < subBLower) {
              return -1
            } else if (subALower > subBLower) {
              return 1
            } else {
              return 0
            }
          })
        }
      })
      Object.values(subscriptionsByStage).forEach(subscriptionsByType => {
        sortSubscriptions(subscriptionsByType)
      })
      return subscriptionsByStage
    }
  },
  mutations: {
    [subscriptionMuts.SET_SUBSCRIPTIONS] (state, subscriptions) {
      state.subscriptions = subscriptions.reduce((acc, curr) => {
        acc[curr['pk']] = curr
        return acc
      }, {})
    },
    [subscriptionMuts.SET_SUBSCRIPTION] (state, subscription) {
      Vue.set(state.subscriptions, subscription.pk, subscription)
    },
    [subscriptionMuts.SET_ACTIVE_SUBSCRIPTION_PK] (state, subscriptionPk) {
      state.activeSubscriptionPk = subscriptionPk
    },
    [subscriptionMuts.SET_ASSIGNMENT_QUEUE] (state, queue) {
      state.assignmentQueue = queue
    },
    [subscriptionMuts.ADD_ASSIGNMENT_TO_QUEUE] (state, assignment) {
      state.assignmentQueue.push(assignment)
    },
    [subscriptionMuts.POP_ASSIGNMENT_FROM_QUEUE] (state) {
      state.assignmentQueue.shift()
    },
    [subscriptionMuts.RESET_STATE] (state) {
      Object.assign(state, initialState())
    }
  },
  actions: {
    subscribeTo: async function ({commit, dispatch, getters}, {type, key, stage}) {
      try {
        // don't subscribe to type, key, stage combinations if they're already present
        let subscription = getters.getSubscriptionsGroupedByType[stage][type].find(elem => {
          if (type === 'random') {
            return true
          }
          return elem.query_key === key
        })
        subscription = subscription || await api.subscribeTo(type, key, stage)
        commit(subscriptionMuts.SET_SUBSCRIPTION, subscription)
        return subscription
      } catch (err) {
        console.log(err)
        handleError(err, dispatch, 'Subscribing unsuccessful')
      }
    },
    async getSubscriptions ({commit, dispatch}) {
      try {
        const subscriptions = await api.fetchSubscriptions()
        commit(subscriptionMuts.SET_SUBSCRIPTIONS, subscriptions)
        return subscriptions
      } catch (err) {
        console.log(err)
        handleError(err, dispatch, 'Unable to fetch subscriptions')
      }
    },
    /**
     * Creates as many assignments as needed to reach MAX_NUMBER_OF_ASSIGNMENTS
     * @param numOfAssignments Use to override default behaviour of creating MAX_NUMBER_OF_ASSIGNMENTS - assignmentQueue.length assignments
     * @returns {Promise<[Promise]>} returns Promise of Array of Promises of assignments
     */
    async getAssignmentsForActiveSubscription ({commit, state, dispatch, getters}, numOfAssignments) {
      numOfAssignments = numOfAssignments || MAX_NUMBER_OF_ASSIGNMENTS - state.assignmentQueue.length
      let assignmentsPromises = []
      for (let i = 0; i < numOfAssignments; i++) {
        assignmentsPromises.push(api.createAssignment({subscription: getters.activeSubscription}))
      }
      return assignmentsPromises
    },
    async deleteAssignment ({commit, state, dispatch}, assignment) {
      try {
        return await api.deleteAssignment({assignment})
      } catch (err) {
        handleError(err, dispatch, 'Unable to delete assignment')
      }
    },
    async cleanAssignmentsFromSubscriptions ({commit, state, dispatch}, excludeActive = true) {
      Object.values(state.subscriptions).forEach(subscription => {
        if (!excludeActive || subscription.pk !== state.activeSubscriptionPk) {
          subscription.assignments.forEach(assignment => {
            api.deleteAssignment({assignment})
          })
        }
      })
    },
    async skipAssignment ({commit, state, dispatch}) {
      try {
        dispatch('deleteAssignment', state.assignmentQueue[0])
          .then(() => {
            // pass numOfAssignments = 1 to create 1 new assignment although maybe two are already in the queue,
            // this is needed because otherwise the current assignment in the comp. might be unknown for a period
            // which will result get incorrectly interpreted as a an ended subscription
            return dispatch('getAssignmentsForActiveSubscription', 1)
          }).then(([promise]) => {
            promise.then(assignment => {
              commit(subscriptionMuts.ADD_ASSIGNMENT_TO_QUEUE, assignment)
              commit(subscriptionMuts.POP_ASSIGNMENT_FROM_QUEUE)
            })
          })
      } catch (err) {
        handleError(err, dispatch, 'Unable to skip assignment')
      }
    },
    async deleteActiveAssignments ({commit, state, dispatch}) {
      try {
        Promise.all(state.assignmentQueue.map(assignment => {
          return dispatch('deleteAssignment', assignment)
        }))
      } catch (err) {
        handleError(err, dispatch, 'Unable to remove assignments.')
      }
    },
    async changeActiveSubscription ({commit, state, dispatch}, subscriptionPk = '') {
      if (subscriptionPk !== state.activeSubscriptionPk) {
        await dispatch('deleteActiveAssignments')
        commit(subscriptionMuts.SET_ACTIVE_SUBSCRIPTION_PK, subscriptionPk)
        let assignmentsPromises = await dispatch('getAssignmentsForActiveSubscription', MAX_NUMBER_OF_ASSIGNMENTS)
        let createdAssignments = []
        for (let promise of assignmentsPromises) {
          try {
            createdAssignments.push(await promise)
          } catch (_) {}
        }
        commit(subscriptionMuts.SET_ASSIGNMENT_QUEUE, createdAssignments)
      }
    },
    async removeActiveSubscription ({commit, state, dispatch}) {
      await dispatch('deleteActiveAssignments')
      commit(subscriptionMuts.SET_ASSIGNMENT_QUEUE, [])
      commit(subscriptionMuts.SET_ACTIVE_SUBSCRIPTION_PK, '')
    },
    async subscribeToType ({commit, state, dispatch, getters}, type) {
      switch (type) {
        case 'random':
          return getters.availableStages.map(stage => {
            dispatch('subscribeTo', {type, stage})
          })
        case 'exam':
          if (getters.isReviewer) {
            const stageKeyCartesian = cartesian(
              getters.availableStages, getters.availableExamTypeQueryKeys)
            return stageKeyCartesian.map(([stage, key]) => {
              dispatch('subscribeTo', {stage, type, key})
            })
          }
          return []
        case 'submission_type':
          const stageKeyCartesian = cartesian(
            getters.availableStages, getters.availableSubmissionTypeQueryKeys)
          return stageKeyCartesian.map(([stage, key]) => {
            dispatch('subscribeTo', {stage, type, key})
          })
      }
    },
    subscribeToAll: once(async ({commit, state, dispatch, getters}) => {
      return Promise.all(flatten(getters.availableTypes.map(type => {
        return dispatch('subscribeToType', type)
      })))
    })
  }
}

export default subscriptions
