
export function nameSpacer (namespace) {
  return function (commitType) {
    return namespace + commitType
  }
}

export function getObjectValueByPath (obj, path) {
  // credit: http://stackoverflow.com/questions/6491463/accessing-nested-javascript-objects-with-string-key#comment55278413_6491621
  if (!path || path.constructor !== String) return
  path = path.replace(/\[(\w+)\]/g, '.$1') // convert indexes to properties
  path = path.replace(/^\./, '') // strip a leading dot
  const a = path.split('.')
  for (let i = 0, n = a.length; i < n; ++i) {
    const k = a[i]
    if (obj instanceof Object && k in obj) {
      obj = obj[k]
    } else {
      return
    }
  }
  return obj
}

/**
 * Use this method to generate a computed property accessing the store for a Vue instance.
 * The get method will return the value at this.$store.state.<path>.
 * The set method will commit the passed value as payload for the specified mutation
 * @param path to the store variable/object being returned
 * @param mutation name of mutation type
 * @param namespace to prepend the mutation type with
 * @returns {*}
 */
export function createComputedGetterSetter ({path, mutation, namespace}) {
  return {
    get () {
      return getObjectValueByPath(this.$store.state, path)
    },
    set (val) {
      this.$store.commit(`${namespace ? namespace + '/' : ''}${mutation}`, val)
    }
  }
}

/**
 * Returns an object of generated computed getter/setter pairs.
 * Can be used to quickly bind a stores state and corresponding setters to a vue component
 * @param namespace optional namespace of mutations
 * @param pathPrefix if set, all items path will be prepended by the path prefix
 * @param items array that contains objects {name, path, mutation}
 */
export function mapStateToComputedGetterSetter ({namespace = '', pathPrefix = '', items = []}) {
  return items.reduce((acc, curr) => {
    // if no path is give, use name
    const itemPath = curr.path || curr.name
    const path = pathPrefix ? `${pathPrefix}.${itemPath}` : itemPath
    acc[curr.name] = createComputedGetterSetter({...curr, path, namespace})
    return acc
  }, {})
}

// thanks to rsp
// https://stackoverflow.com/questions/12303989/cartesian-product-of-multiple-arrays-in-javascript/43053803#43053803
let cartesianHelper = (a, b) => [].concat(...a.map(a => b.map(b => [].concat(a, b))))
export function cartesian (a, b, ...c) {
  return b ? cartesian(cartesianHelper(a, b), ...c) : a
}

// flatten an array
export function flatten (list) {
  return list.reduce(
    (a, b) => a.concat(Array.isArray(b) ? flatten(b) : b), []
  )
}

export function objectifyArray (arr, key = 'pk') {
  return arr.reduce((acc, curr) => {
    acc[curr[key]] = curr
    return acc
  }, {})
}

export function once (fn, context) {
  let result
  return function () {
    if (!result) {
      result = fn.apply(context || this, arguments)
    }
    return result
  }
}

export function handleError (err, dispatch, fallbackMsg) {
  if (err.response) {
    if (err.response.status === 401) {
      dispatch('logout', 'You\'ve been logged out')
    } else {
      throw err
    }
  } else {
    if (fallbackMsg) {
      throw new Error(fallbackMsg)
    }
  }
}
